(() => {
	var CanvasLib = new PlugIn.Library(new Version("1.2"));

	CanvasLib.getCanvasNames = function(){
		return canvases.map(function(canvas){return canvas.name})
	}
	
	CanvasLib.sortCanvasesByName = function(){
		if (canvases.length == 1){return true}
		// get canvas names
		canvasNames = canvases.map(function(canvas){return canvas.name})
		// sort the list and reverse the order
		sortedNames = canvasNames.sort().reverse()
		// iterate the reversed list of names
		sortedNames.forEach(function(canvasName){
			// locate canvas with matching name
			for(i = 0; i < canvases.length; i++){
				if (canvases[i].name.localeCompare(canvasName) == 0){
					targetCanvas = canvases[i]
					break
				}
			}
			// if canvas is not top canvas, move to top
			if (targetCanvas.id != canvases[0].id){
				targetCanvas.orderBefore(canvases[0])
			}
		})
	}
	
	CanvasLib.deleteAllCanvases = function(showAlert){
		if (canvases.length > 1){
			if (showAlert == true){
				var alert = new Alert("CONFIRMATION REQUIRED", "Are you sure you want to remove all but the selected canvas?")
				alert.addOption("Cancel")
				alert.addOption("Remove")
				alert.show(function(result) {
					if (result == 0) {
						console.log("user cancelled")
					} else {
						cnvs = document.windows[0].selection.canvas
						if(cnvs.id != canvases[0].id){
							cnvs.orderBefore(canvases[0])
						}
						for(var i = canvases.length; i--;) { 
							if(i != 0){canvases[i].remove()}
						}
					}
				})
			} else {
				cnvs = document.windows[0].selection.canvas
				if(cnvs.id != canvases[0].id){
					cnvs.orderBefore(canvases[0])
				}
				for(var i = canvases.length; i--;) { 
					if(i != 0){canvases[i].remove()}
				}
			}
		}
	}
	
	CanvasLib.addCanvasForEveryMonth = function(showAlert){
		if(showAlert){
			var alert = new Alert("Confirmation Required", "Add a canvas for each month of the year?")
			alert.addOption("Continue")
			alert.addOption("Stop")
			alert.show(function(result){
				if (result == 1){
					throw new Error('script cancelled')
				} else {
					locale = "en-us"
					currentYear = new Date().toLocaleString(locale, { year: "numeric"})
					mths = new Array()
					for (i = 1; i < 12; i++) { 
						var date = new Date(i + "/1/" + currentYear),
						month = date.toLocaleString(locale, { month: "long" })
						mths.push(month)
					}
					sze = document.windows[0].selection.canvas.size
					canvasNames = this.getCanvasNames()
					mths.forEach(function(mth){
						if(canvasNames.indexOf(mth) === -1){
							cnvs = addCanvas()
							cnvs.name = mth
							cnvs.size = sze
						}
					})
				}
			})
		} else {
			locale = "en-us"
			currentYear = new Date().toLocaleString(locale, { year: "numeric"})
			mths = new Array()
			for (i = 1; i < 12; i++) { 
				var date = new Date(i + "/1/" + currentYear),
				month = date.toLocaleString(locale, { month: "long" })
				mths.push(month)
			}
			sze = document.windows[0].selection.canvas.size
			canvasNames = this.getCanvasNames()
			mths.forEach(function(mth){
				if(canvasNames.indexOf(mth) === -1){
					cnvs = addCanvas()
					cnvs.name = mth
					cnvs.size = sze
				}
			})
		}
	}
	
	CanvasLib.getNamedCanvas = function(canvasName){
		for(i = 0; i < canvases.length; i++){
			if (canvases[i].name.localeCompare(canvasName) == 0){
				return canvases[i]
			}
		}
		errorString = "There is no canvas named: " + canvasName
		new Alert('ERROR', errorString).show(function(result){})
		throw new Error(errorString)
	}
	
	// DIMENSIONS
	
	CanvasLib.newCanvasWithDimensions = function(cWpt, cHpt){
		cnvs = addCanvas()
		cnvs.size = new Size(cWpt, cHpt)
		return cnvs
	}
	
	CanvasLib.setDimensionsOfEveryCanvasTo = function(cWpt, cHpt){
		var newSize = new Size(cWpt, cHpt) 
		canvases.forEach(function(canvas){
			canvas.size = newSize
		})
	}
	
	CanvasLib.setDimensionsOfCurrentCanvasTo = function(cWpt, cHpt){
		cnvs = document.windows[0].selection.canvas
		cnvs.size = new Size(cWpt, cHpt)
	}
	
	CanvasLib.setDimensionsOfCanvasTo = function(cnvs, cWpt, cHpt){
		cnvs.size = new Size(cWpt, cHpt)
	}
	
	CanvasLib.swapDimensionsOfCanvas = function(cnvs){
		cWpt = cnvs.size.width
		cHpt = cnvs.size.height
		cnvs.size = new Size(cHpt, cWpt)
	}
	
	// SET SIZING MODE TO FIXED
	
	CanvasLib.setSizingModeOfCanvasToFixed = function(cnvs){
		cnvs.canvasSizingMode = CanvasSizingMode.Fixed
	}
	
	CanvasLib.setSizingModeOfCurrentCanvasToFixed = function(){
		cnvs = document.windows[0].selection.canvas
		cnvs.canvasSizingMode = CanvasSizingMode.Fixed
	}
	
	CanvasLib.setSizingModeOfEveryCanvasToFixed = function(){
		canvases.forEach(function(canvas){
			canvas.canvasSizingMode = CanvasSizingMode.Fixed
		})
	}
	
	// COPY SIZING PARAMETERS TO ALL
	
	CanvasLib.setSizingParamsOfEveryCanvas = function(cnvs){
		console.log('setSizingParamsOfEveryCanvas(cnvs)')
		cnvsSizingMode = cnvs.canvasSizingMode
		cnvsSize = cnvs.size
		cnvsAutosizesUp = cnvs.autosizesUp
		cnvsAutosizesDown = cnvs.autosizesDown
		cnvsAutosizesLeft = cnvs.autosizesLeft
		cnvsAutosizesRight = cnvs.autosizesRight
		canvases.forEach(function(canvas){
			canvas.canvasSizingMode = cnvsSizingMode
			canvas.size = cnvsSize
			canvas.autosizesUp = cnvsAutosizesUp
			canvas.autosizesDown = cnvsAutosizesDown
			canvas.autosizesLeft = cnvsAutosizesLeft
			canvas.autosizesRight = cnvsAutosizesRight
		})
	}
	
	// GET CURRENT CANVAS
	
	CanvasLib.getCurrentCanvas = function(){
		cnvs = document.windows[0].selection.canvas
		return cnvs
	}
	
	// MOVE CANVAS TO TOP|BOTTOM
	
	CanvasLib.moveCanvasToTop = function(cnvs){
		if (canvases.length > 1){
			if(cnvs.id != canvases[0].id){
				cnvs.orderBefore(canvases[0])
			}
		}	
	}
	
	CanvasLib.moveCanvasToBottom = function(cnvs){
		if (canvases.length > 1){
			if(cnvs.id != canvases[canvases.length-1].id){
				cnvs.orderAfter(canvases[canvases.length-1])
			}
		}
	}
	
	CanvasLib.deleteEveryCanvasExcept= function(cnvs){
		if (canvases.length > 1){
			if(cnvs.id != canvases[0].id){
				cnvs.orderBefore(canvases[0])
			}
			for(var i = canvases.length; i--;) { 
				if(i != 0){canvases[i].remove()}
			}
		}
	}
	
	CanvasLib.deleteEveryCanvasExceptCurrent= function(){
		cnvs = document.windows[0].selection.canvas
		if (canvases.length > 1){
			if(cnvs.id != canvases[0].id){
				cnvs.orderBefore(canvases[0])
			}
			for(var i = canvases.length; i--;) { 
				if(i != 0){canvases[i].remove()}
			}
		}
	}
	
	CanvasLib.renameEveryCanvasToMatchOrder = function(basename,separator){
		cvss = canvases.reverse()
		for(i = 0; i < cvss.length; i++){
			index = i + 1
			canvases[i].name = basename + separator + index
		}
	}
	
	CanvasLib.setAutoSizingOfCurrentCanvas = function(trueFalse){
		cnvs = document.windows[0].selection.canvas
		cnvs.autosizesUp = cnvs.autosizesDown = cnvs.autosizesLeft = cnvs.autosizesRight = trueFalse
	}
	
	CanvasLib.setAutoSizingOfEveryCanvas = function(trueFalse){
		for(i = 0; i < canvases.length; i++){
			cnvs = canvases[i]
			cnvs.autosizesUp = cnvs.autosizesDown = cnvs.autosizesLeft = cnvs.autosizesRight = trueFalse
		}
	}
	
	CanvasLib.setSizeOfCanvasToMatchPageSize = function(cnvs){
		cnvs.canvasSizingMode = CanvasSizingMode.Fixed
		cnvs.size = new Size(cnvs.pageSize.width, cnvs.pageSize.height)
	}
	
	CanvasLib.setSizeOfCurrentCanvasToMatchPageSize = function(){
		cnvs = document.windows[0].selection.canvas
		cnvs.canvasSizingMode = CanvasSizingMode.Fixed
		cnvs.size = new Size(cnvs.pageSize.width, cnvs.pageSize.height)
	}
	
	CanvasLib.setSizeOfEveryCanvasToMatchPageSize = function(){
		canvases.forEach(function (cnvs){
			cnvs.canvasSizingMode = CanvasSizingMode.Fixed
			cnvs.size = new Size(cnvs.pageSize.width, cnvs.pageSize.height)
		})
	}
			
	// returns contents of matching strings file
	CanvasLib.documentation = function(){
		// get the plugin
		var parentPlugin = CanvasLib.plugIn
		// get the url for the text file inside this plugin
		var url = parentPlugin.resourceNamed("CanvasLib.strings")
		// read the file
		url.fetch(function(data){
			dataString = data.toString()
			console.log(dataString) // show in console
			return dataString
		})
	}
	
   return CanvasLib;
})();